@@ -23,10 +23,13 @@
 
 #include <sys/types.h>
 #include <kio/slavebase.h>
-#include <QtCore/QList>
+#include <QList>
+#include <QLoggingCategory>
 
 #include <KCddb/Cdinfo>
 
+Q_DECLARE_LOGGING_CATEGORY(AUDIOCD_KIO_LOG)
+
 class KConfigSkeleton;
 using namespace KCDDB;
 
@@ -41,12 +44,12 @@ public:
   explicit AudioCDEncoder(KIO::SlaveBase *slave) : ioslave(slave) {}
   
   /**
-   * Deconstructor.
+   * Destructor.
    */
   virtual ~AudioCDEncoder(){}
   
   /**
-   * Initiallizes the decoder, loading libraries, etc.  Encoders
+   * Initializes the decoder, loading libraries, etc.  Encoders
    * that don't return true will will deleted and not used.
    * @returns false if unable to initialize the encoder.
    */
@@ -60,7 +63,7 @@ public:
   /**
    * Helper function to determine the end size of a
    * encoded file.
-   * @param time_secs the lengh of the audio track in seconds.
+   * @param time_secs the length of the audio track in seconds.
    * @returns the size of a file if it is time_secs in length.
    */ 
   virtual unsigned long size(long time_secs) const = 0;
@@ -102,12 +105,12 @@ public:
   
   /**
    * Passes a little bit of cd data to be encoded
-   * This function is most likly called many many times.
+   * This function is most likely called many many times.
    * @param buf pointer to the audio that has been read in so far
-   * @param frames the number of frames of audio that are in buf
+   * @param frames the number of frames of audio that are in @p buf
    * @return size of the data that was created by this function, -1 on error.
    */
-  virtual long read(int16_t * buf, int frames) = 0;
+  virtual long read(qint16 * buf, int frames) = 0;
 
   /**
    * Perform any final file creation/padding that is necessary
@@ -119,7 +122,7 @@ public:
    * Returns a configure widget for the encoder
    */ 
   virtual QWidget* getConfigureWidget(KConfigSkeleton** manager) const
-                   { Q_UNUSED(manager); return NULL; } 
+                   { Q_UNUSED(manager); return nullptr; }
  
   /**
    * Returns the last error message; called when e.g. read() returns -1.
@@ -128,7 +131,7 @@ public:
 
   /**
    * Helper function to load all of the AudioCD Encoders from libraries.
-   * Uses KStandardDirs to find where libraries could be, opens all of the ones
+   * Uses QLibraryInfo to find where libraries could be, opens all of the ones
    * that we might own audiocd_encoder_* and then uses the symbol 
    * create_audiocd_encoders to obtain the encoders from that library.
    * @param slave ioslave needed if the plugin is going to be used to encode something.
