@@ -30,13 +30,13 @@
 #include "audiocd_flac_encoder.h"
 #endif
 
-#include <kconfig.h>
+#include <KConfig>
 #include "audiocd_kio_debug.h"
 
 #include <QDateTime>
 #include <QPair>
 
-Q_LOGGING_CATEGORY(AUDIOCD_KIO_LOG, "kf5.kio.audiocd")
+Q_LOGGING_CATEGORY(AUDIOCD_KIO_LOG, "kf.kio.slaves.audiocd")
 
 extern "C"
 {
@@ -104,7 +104,7 @@ static FLAC__SeekableStreamEncoderSeekStatus  SeekCallback(const FLAC__SeekableS
 EncoderFLAC::EncoderFLAC(KIO::SlaveBase *slave) : AudioCDEncoder(slave) {
     d = new Private();
     d->ioslave = slave;
-    d->encoder = 0;
+    d->encoder = nullptr;
 #if defined(FLAC_API_VERSION_CURRENT) && FLAC_API_VERSION_CURRENT > 7
     d->compression_level = 5;
 #endif
@@ -127,7 +127,7 @@ QWidget* EncoderFLAC::getConfigureWidget(KConfigSkeleton** manager) const {
 
 bool EncoderFLAC::init(){
     d->encoder = FLAC__stream_encoder_new();
-    d->metadata = 0;
+    d->metadata = nullptr;
     d->data = 0;
     return true;
 }
@@ -172,7 +172,7 @@ long EncoderFLAC::readInit(long size) {
 #if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
     FLAC__stream_encoder_init(d->encoder);
 #else
-    FLAC__stream_encoder_init_stream(d->encoder, WriteCallback, NULL, NULL, MetadataCallback, d);
+    FLAC__stream_encoder_init_stream(d->encoder, WriteCallback, nullptr, nullptr, MetadataCallback, d);
 #endif
     return d->data;
 }
@@ -197,7 +197,7 @@ long EncoderFLAC::readCleanup()
     if (d->metadata) {
 	 FLAC__metadata_object_delete(d->metadata[0]);
          delete[] d->metadata;
-	 d->metadata = 0;
+	 d->metadata = nullptr;
     }
     return 0;
 }
@@ -218,20 +218,23 @@ void EncoderFLAC::fillSongInfo( KCDDB::CDInfo info, int track, const QString &co
 				Comment(QLatin1String( "COMMENT" ), comment),
 				Comment(QLatin1String( "DATE" ), QVariant(QString()) )};
     if (info.get(Year).toInt() > 0) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+	const QDateTime dt = QDate(info.get(Year).toInt(), 1, 1).startOfDay();
+#else
 	QDateTime dt(QDate(info.get(Year).toInt(), 1, 1));
+#endif
 	comments[6] = Comment(QLatin1String( "DATE" ), dt.toString(Qt::ISODate));
     }
 
     FLAC__StreamMetadata_VorbisComment_Entry entry;
     QString field;
-    QByteArray cfield;
     int num_comments = 0;
 
     for(int i=0; i<7; i++) {
 	if (!comments[i].second.toString().isEmpty()) {
 	    field = comments[i].first+QLatin1Char( '=' )+comments[i].second.toString();
-	    cfield = field.toUtf8();
-	    entry.entry = (FLAC__byte*)qstrdup(cfield);
+	    const QByteArray cfield = field.toUtf8();
+	    entry.entry = (FLAC__byte*)qstrdup(cfield.data());
 	    entry.length = cfield.length();
 	    // Insert in vorbiscomment and assign ownership of pointers to FLAC
 	    FLAC__metadata_object_vorbiscomment_insert_comment(d->metadata[0], num_comments, entry, false);
