@@ -31,6 +31,7 @@
 #include <KConfig>
 #include <QByteArray>
 #include <QDateTime>
+#include <QRandomGenerator>
 
 extern "C"
 {
@@ -147,7 +148,7 @@ long EncoderVorbis::flush_vorbis(void) {
   long processed(0);
 
   while(vorbis_analysis_blockout(&d->vd,&d->vb)==1) {
-    vorbis_analysis(&d->vb,NULL);
+    vorbis_analysis(&d->vb,nullptr);
     /* Non-ancient case.  */
     vorbis_bitrate_addblock(&d->vb);
 
@@ -211,8 +212,7 @@ long EncoderVorbis::readInit(long /*size*/){
   vorbis_analysis_init(&d->vd,&d->vi);
   vorbis_block_init(&d->vd,&d->vb);
 
-  qsrand(time(NULL));
-  ogg_stream_init(&d->os,qrand());
+  ogg_stream_init(&d->os, QRandomGenerator::global()->generate());
 
   vorbis_analysis_headerout(&d->vd,&d->vc,&header,&header_comm,&header_code);
 
@@ -280,7 +280,11 @@ void EncoderVorbis::fillSongInfo( KCDDB::CDInfo info, int track, const QString &
   commentFields.append(CommentField("COMMENT", comment));
 
   if (info.get(Year).toInt() > 0) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    const QDateTime dt = QDate(info.get(Year).toInt(), 1, 1).startOfDay();
+#else
     QDateTime dt( QDate(info.get(Year).toInt(), 1, 1) );
+#endif
     commentFields.append(CommentField("DATE",QLatin1String( dt.toString(Qt::ISODate).toUtf8().data() )));
   }
 
@@ -289,7 +293,7 @@ void EncoderVorbis::fillSongInfo( KCDDB::CDInfo info, int track, const QString &
     // if the value is not empty
     if(!(*it).second.toString().isEmpty()) {
 
-      char *key = qstrdup((*it).first);
+      char *key = qstrdup((*it).first.constData());
       char *value = qstrdup((*it).second.toString().toUtf8().data());
 
       vorbis_comment_add_tag(&d->vc, key, value);
