@@ -32,7 +32,7 @@
 class EncoderVorbisConfig : public QWidget, public Ui::EncoderVorbisConfig
 {
 public:
-  EncoderVorbisConfig( QWidget *parent = 0 ) : QWidget( parent ) {
+  EncoderVorbisConfig( QWidget *parent = nullptr ) : QWidget( parent ) {
     setupUi( this );
   }
 };
@@ -40,7 +40,7 @@ public:
 
 /**
  * Ogg Vorbis encoder.
- * Check out http://www.vorbis.com/ for lots of information.
+ * Check out https://xiph.org/vorbis/ for more information.
  */
 class EncoderVorbis : public AudioCDEncoder {
 
